/*
 * Create a pipeline, waits for it to finish, returns exit code
 * of last command.
 */

#include <stdio.h>
#include <signal.h>

#define nil(type) ( (type) 0)

#define local static

local int pidindex = 0;

FILE *pipeto();

int pipeline(in, err, cmdv, argvv, out)
FILE *in, *out, *err;
char **cmdv;
char ***argvv;
{
	int pid, status;
	FILE *newin;
	int i;
	int pstatus;

	set_signals();

	for (i = 0; cmdv[i] != nil(char *); i++) {
		if (cmdv[i+1] == nil(char *)) {
			pid = backv(cmdv[i], in, out, err, argvv[i]);
			if (in != stdin) {
				fclose(in);
			}
		} else {
			newin = pipeto(in, err, cmdv[i], argvv[i]);
			fclose(in);
			in = newin;
		}
	}
	pstatus = pipewait();

	reset_signals();

	pidindex = 0;

	pwait(pid, &status);
	status >>= 8;
	return( pstatus | status );
}

/*
 * Create a pipe, invoke the command with given output and pipe output,
 * return pipe input.
 *
 * Save process id's.
 *
 * The "fclose" is important.
 */

#define MAXNCMDS 50

local int pidlist[MAXNCMDS];

FILE *pipeto(in, err, cmd, argv)
FILE *in, *err;
char *cmd;
char **argv;
{
	FILE *fp[2];

	fpipe(fp);
	pidlist[pidindex++] = backv(cmd, in, fp[1], err, argv);
	fclose(fp[1]);
	return(fp[0]);
}

/*
 * Wait for all processes created via "pipeto".
 */

pipewait()
{
	int i;
	int status;
	int total_status = 0;

	for (i = 0; i < pidindex; i++) {
	    pwait(pidlist[i], &status);
	    if (status != 0) {
		kill_pipe();
	    }
	    status >>= 8;
	    total_status |= status;
	}

    return(total_status);
}

/*
 * create a FILE pipe vector
 */

fpipe(fp)
FILE *fp[2];
{
	int fd[2];

	pipe(fd);
	fp[0] = fdopen(fd[0], "r");
	fp[1] = fdopen(fd[1], "w");
	if (fp[0] == nil(FILE *) || fp[1] == nil(FILE *)) {
		panic("fpipe: can't fdopen");
	}
}

kill_pipe()
{
    int i;
    
    for (i = 0; i < pidindex; i++) {
	kill(pidlist[i],SIGTERM);
    }
    exit(1);
}

static int (*hup)();
static int (*intr)();
static int (*term)();
static int (*pipes)();

set_signals()
{
	hup = signal(SIGHUP, kill_pipe);

	intr = signal(SIGINT, kill_pipe);

	term = signal(SIGTERM, kill_pipe);

	pipes = signal(SIGPIPE, kill_pipe);
}

reset_signals()
{
    signal(SIGHUP, hup);
    signal(SIGINT, intr);
    signal(SIGTERM, term);
    signal(SIGPIPE, pipes);
}
