#include "parser.h"
#include "ciftomann.h"
#include "fd.h"
#include <stdio.h>
#include <math.h>

#define HORIZONTAL	0
#define VERTICAL	1
#define IDENTICAL	2
#define NON_MANHATTEN	3

FILE *popen();

extern char *ProgName;
extern char *bin_lib_name;

char sort_cmd[101];

FILE *out_desc;

char *bb_file_name;
char *basename;

static double circle_x1, circle_x2, circle_x3;
static double circle_y1, circle_y2, circle_y3;

double scale_factor;

main(argc,argv) 
int argc;
char **argv;
{
    int grid;

    ProgName = "edger";

    basename = argv[1];
    sscanf(argv[2], "%lf %d", &scale_factor, &grid);
    bb_file_name = argv[3];
    sprintf(sort_cmd,"%s/sort %s", bin_lib_name, argv[4]);

    scale_factor /= grid;

	/* these are a kludge to represent a circle as
	 * three stacked boxes
	 */

    circle_y1 = .25;
    circle_y2 = .71;
    circle_y3 = .96824584;

    circle_x1 = sqrt(1 - circle_y1*circle_y1);
    circle_x2 = sqrt(1 - circle_y2*circle_y2);
    circle_x3 = sqrt(1 - circle_y3*circle_y3);

    parse(stdin);
    write_bb();
    exit(0);
}

EmitEdge(x,xend,y,sense) 
int x,xend,y,sense;
{
    EDGE edge;

    update_bb(x,xend,y);
    edge.x = x;
    edge.y = y;
    edge.sense = sense;
    edge.xend = xend;

    if ( fwrite( &edge, sizeof(edge), 1, out_desc) == 0 ) {
	panic("Bad write in EmitEdge");
    }
}

    /* all of the following routines are called by the cif-parser
     * as each of the various cif constructs are detected
     */

proc_box(length,width,center,dir)
int length,width;
point *center,*dir;
{
    int xmax,ymax,xmin,ymin;

    if (dir != NIL(point)) {
	if (dir->x*dir->y != 0) {
	    error("Non-Manhatten box",center->x,center->y);
	} else if (dir->x == dir->y) {
	    error("Degenerate direction given for box",center->x,center->y);
	} else if (dir->x == 0) {
	    int temp;

	    temp = length;
	    length = width;
	    width = temp;
	}
    }

    xmax = center->x + length/2;
    xmin = center->x - length/2;
    ymax = center->y + width/2;
    ymin = center->y - width/2;

    EmitEdge(xmin,xmax,ymax,DOWN);
    EmitEdge(xmin,xmax,ymin,UP);
}

proc_wire(width,path)
int width;
point *path;
{
    
    point *first,*last;

    last = first = path;

    if (path->next == NIL(point)) {
	proc_roundflash(width,first);
    }

	/* go down the wire's path, processing each segment */

    while (path->next != NIL(point)) {

	path = path->next;

	if (ProcessSegment(last,path,width) == ERROR)
	    return;
	last = path;
    }

}

#define EdgeType(first,last) \
    ( first->x == last->x ? \
	( first->y == last->y ? IDENTICAL : VERTICAL ) :\
	( first->y == last->y ? HORIZONTAL : NON_MANHATTEN ) )


    /* turn each segment of a wire into edges */

ProcessSegment(last,next,Width)
point *last,*next;
{

    int x,xend,miny,maxy;

    switch ( EdgeType(last,next) ) {
	
	case HORIZONTAL :
	    x = Min(last->x,next->x) - Width/2;
	    xend = Max(last->x,next->x) + Width/2;
	    EmitEdge(x,xend,last->y + Width/2,DOWN);
	    EmitEdge(x,xend,last->y - Width/2,UP);
	    break;

	case VERTICAL :
	    x = last->x - Width/2;
	    xend = last->x + Width/2;
	    maxy = Max(last->y,next->y) + Width/2;
	    miny = Min(last->y,next->y) - Width/2;
	    EmitEdge(x,xend,maxy,DOWN);
	    EmitEdge(x,xend,miny,UP);
	    break;

	case IDENTICAL :
	    break;

	case NON_MANHATTEN :
	    error("Non-Manhatten segment in wire\n",next->x,next->y);
	    return(ERROR);
	
    }
    return(GOOD);
}

#define round(x) ( (int) floor( (double) (x) + .5) )

proc_roundflash(diameter,center)
int diameter;
point *center;
{
    double radius = diameter/2.;
    double x1, x2, x3, y1, y2, y3;
    int x_center, y_center;

	/*
	 * as a hack to make circles more symmetric, the center
	 * is pre-rounded to the pattern generater grid, so
	 * that every circle will be similar
	 */

    x_center = round( round( scale_factor*center->x )/scale_factor );
    y_center = round( round( scale_factor*center->y )/scale_factor );

    x1 = circle_x1*radius;
    x2 = circle_x2*radius;
    x3 = circle_x3*radius;
    y1 = circle_y1*radius;
    y2 = circle_y2*radius;
    y3 = circle_y3*radius;

    EmitEdge(x_center - (int) x3, x_center + (int) x3,
	     y_center + (int)y3, DOWN);

    EmitEdge(x_center - (int) x2, x_center - (int) x3,
	     y_center + (int)y2, DOWN);
    EmitEdge(x_center + (int) x3, x_center + (int) x2,
	     y_center + (int)y2, DOWN);

    EmitEdge(x_center - (int) x1, x_center - (int) x2,
	     y_center + (int)y1, DOWN);
    EmitEdge(x_center + (int) x2, x_center + (int) x1,
	     y_center + (int)y1, DOWN);

    EmitEdge(x_center - (int) x1, x_center - (int) x2,
	     y_center - (int)y1, UP);
    EmitEdge(x_center + (int) x2, x_center + (int) x1,
	     y_center - (int)y1, UP);

    EmitEdge(x_center - (int) x2, x_center - (int) x3,
	     y_center - (int)y2, UP);
    EmitEdge(x_center + (int) x3, x_center + (int) x2,
	     y_center - (int)y2, UP);

    EmitEdge(x_center - (int) x3, x_center + (int) x3,
	     y_center - (int)y3, UP);
}

proc_call(cell_num)
int cell_num;
{
    panic("Call command found in allegedly flattened cif");
}

proc_defstart(cell_num,a,b)
int cell_num,a,b;
{
    panic("Definition Start command found in allegedly flattened cif");
}

proc_deffinish()
{
    panic("Definition Finish command found in allegedly flattened cif");
}

proc_defdelete(cell_num)
int cell_num;
{
    panic("Definition Delete command found in allegedly flattened cif");
}

proc_userextension(extension_num)
int extension_num;
{
    panic("user extension command found in allegedly flattened cif");
}

proc_end()
{
    if ( pclose(out_desc) != 0 ) {
	fprintf(stderr,"Panic in edger : bad return status from sort\n");
	exit(1);
    }
    
}

    /*
     *  a new layer has been entered, so close off the old output file
     *  and start up a new sort process to sort this layer's edges
     */

proc_layer(name)
char *name;
{
    char file[15];
    int desc;

    sprintf(file,"%s.%s",basename,name);

    if (out_desc != NULL) {
	int status;
	status = pclose(out_desc);
	if (status != 0) {
	    fprintf(stderr,"Panic in edger : bad return status from sort\n");
	    exit(1);
	}
    }

    if ( (desc = creat(file,0644)) < 0 ) {
	fprintf(stderr,"Panic in edger : cannot open temp file %s\n",
		file);
	exit(1);
    }

    dup2(desc,STDOUT);
    close(desc);

    if ( (out_desc = popen(sort_cmd,"w")) == NULL) {
	fprintf(stderr,"Panic in edger : cannot fork off the sorter\n");
	exit(1);
    }

}

int minx, miny, maxx, maxy;
int first = 0;

#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) > (y) ? (x) : (y))

    /* I am depending on x < xend */

update_bb(x,xend,y)
int x,xend,y;
{
    int small;

    if (first == 0) {
	minx = x;
	maxx = xend;
	miny = maxy = y;
	first = 1;
    }

    minx = min(minx,x);
    maxx = max(maxx,xend);
    miny = min(miny,y);
    maxy = max(maxy,y);
}

write_bb()
{
    FILE *bb_file;
    int left,bottom,right,top;

    if ( (bb_file = fopen(bb_file_name,"w")) == NULL) {
	fprintf("Panic in edger : could not open temp file %s\n",
	    bb_file_name);
	exit(1);
    }

    fprintf(bb_file,"%d %d %d %d\n", minx, miny, maxx, maxy);
    fclose(bb_file);
}

error(str,x,y)
char *str;
int x,y;
{
    fprintf(stderr,"Error in flattened cif : %s at (%d,%d)\n",str,x,y);
}
