#include "parser.h"

boolean in_definition = FALSE;

    /*
     * this is a parser for CIF, it calls routines proc_box, proc_wire,
     * etc, as the various cif objects are recognized. The parser is
     * in the style of a recursive descent type parser, except cif
     * doesn't require recursion because of its' syntax.
     */
	
parse(input_file)
FILE *input_file;	/* input_file is the already open file
			 * containing the cif
			 */
{
    
    infile = input_file;

    for(;;) {

	flush(BLANK);

	if ( IS(DIGIT,GET_CHAR) ) {

	    int user_num;

	    ungetc(current_char,infile);

	    user_num = get_integer();

	    proc_userextension(user_num);

	} else {
	    
	    switch( current_char ) {

		case 'E' :
		    {
			int c;

			while ( (c = getchar()) != EOF && IS(BLANK,(char) c) ) 
			    ;
			
			if ( c != EOF ) {
			    error("Extra characters after End statement");
			}

			proc_end();

			return;
		    }
		
		case 'P' :
		    {
			point *path;

			if ( (path = get_path()) == NIL(point) ) {
			    error("Cannot find path in Polygon command");
			}
			proc_polygon(path);

			while ( path != NIL(point) ) {
			    FREE(path);
			    path = path->next;
			}
		    }
		    break;

		case 'B' :
		    {
			int width,length;
			point *center;
			point *dir;

			if ( (length = get_integer()) == INFINITE ) {
			    error("Cannot find length in Box command");
			}

			if ( (width = get_integer()) == INFINITE ) {
			    error("Cannot find width in Box command");
			}

			if ( (center = get_point()) == NIL(point) ) {
			    error("Cannot find center in Box command");
			}

			dir = get_point();

			proc_box(length,width,center,dir);

			FREE(center);
			FREE(dir);

		    }
		    break;
		
		case 'R' :
		    {
			int diameter;
			point *center;

			if ( (diameter = get_integer()) == INFINITE ) {
			    error("Cannot find diameter in RoundFlash command");
			}

			if ( (center = get_point()) == NIL(point) ) {
			    error("Cannot find center in RoundFlash command");
			}

			proc_roundflash(diameter,center);

			FREE(center);
		    }
		    break;

		case 'W' :
		    {
			int width;
			point *path;

			if ( (width = get_integer()) == INFINITE ) {
			    error("Cannot find width in Wire command");
			}

			if ( (path = get_path()) == NIL(point) ) {
			    error("Cannot find path in Polygon command");
			}
			
			proc_wire(width,path);

			while ( path != NIL(point) ) {
			    FREE(path);
			    path = path->next;
			}
		    }
		    break;

		case 'L' :
		    {
			char *name;

			flush(BLANK);

			if ( (name = get_shortname()) == NIL(char) ) {
			    error("Cannot find name in Layer command");
			}

			proc_layer(name);

			FREE(name);
		    }
		    break;

		case 'D' :
		    flush(BLANK);

		    switch ( GET_CHAR ) {
			
			case 'S' :
			    {
				int cell_num;
				int a,b;

				if ( in_definition ) {
				    error("Nested cell definitions not allowed");
				}

				in_definition = TRUE;

				if ( (cell_num = get_integer()) == INFINITE ) {
				    error("Cannot find cell number in Definition Start command");
				}

				if ( (a = get_integer()) == INFINITE ) {
				    a = b = 1;
				} else {
				    if ( (b = get_integer()) == INFINITE ) {
					error("Cannot find both scale factors in Definition Start command");
				    }
				}

				proc_defstart(cell_num,a,b);
			    }
			    break;
			    
			case 'F' :

			    {
				if ( !in_definition ) {
				    error(" Definition Finish found outside of cell definition");
				}

				proc_deffinish();
			    }

			    in_definition = FALSE;
			    break;

			case 'D' :
			    {
				int cell_num;

				if ( !in_definition ) {
				    error(" Definition Finish found outside of cell definition");
				}

				if ( (cell_num = get_integer()) == INFINITE ) {
				    error("Cannot find cell_num in Definition Delete command");
				}

				proc_defdelete(cell_num);
			    }
			    break;

			default :
			    {
				error("Unrecogizable command");
			    }
		    }
		    break;

		case 'C' :
		    {
			int cell_num;

			if ( (cell_num = get_integer()) == INFINITE ) {
			    error("Cannot find cell number in Call command");
			}

			get_transform();

			proc_call(cell_num);
		    }
		    break;
		
		case '(' :
		    {
			char c;
			int level = 1;

			while (level > 0 ) {
			    GET_CHAR;
			    if (current_char == '(') 
				level++;
			    else if (current_char == ')')
				level--;
			}

		    }
		    break;

		default :
		    {
			error("Unrecognizable command");
		    }

	    }/* switch */
	}

	flush(BLANK);

	if (GET_CHAR != ';') {
	    error("Bad end of command found");
	}

    } /* for */
} /* parse */
